@@ -29,19 +29,17 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
-// Note: This header defines a simplication of Poco::MetaObject
-// that allows us to tag MetaObjects with an associated library name.
+// Note: This header defines a simplication of a shared library
 
 #ifndef CLASS_LOADER__META_OBJECT_HPP_
 #define CLASS_LOADER__META_OBJECT_HPP_
 
-#include <console_bridge/console.h>
-#include "class_loader/visibility_control.hpp"
-
-#include <typeinfo>
 #include <string>
+#include <typeinfo>
 #include <vector>
 
+#include "class_loader/visibility_control.hpp"
+
 namespace class_loader
 {
 
@@ -50,7 +48,7 @@ class ClassLoader;  // Forward declaration
 namespace impl
 {
 
-typedef std::vector<class_loader::ClassLoader *> ClassLoaderVector;
+class AbstractMetaObjectBaseImpl;
 
 /**
  * @class AbstractMetaObjectBase
@@ -62,7 +60,10 @@ public:
   /**
    * @brief Constructor for the class
    */
-  AbstractMetaObjectBase(const std::string & class_name, const std::string & base_class_name);
+  AbstractMetaObjectBase(
+    const std::string & class_name,
+    const std::string & base_class_name,
+    const std::string & typeid_base_class_name = "UNSET");
   /**
    * @brief Destructor for the class. THIS MUST NOT BE VIRTUAL AND OVERRIDDEN BY
    * TEMPLATE SUBCLASSES, OTHERWISE THEY WILL PULL IN A REDUNDANT METAOBJECT
@@ -72,57 +73,75 @@ public:
 
   /**
    * @brief Gets the literal name of the class.
+   *
    * @return The literal name of the class as a C-string.
    */
-  std::string className() const;
+  const std::string & className() const;
 
   /**
    * @brief gets the base class for the class this factory represents
    */
-  std::string baseClassName() const;
+  const std::string & baseClassName() const;
   /**
    * @brief Gets the name of the class as typeid(BASE_CLASS).name() would return it
    */
-  std::string typeidBaseClassName() const;
+  const std::string & typeidBaseClassName() const;
 
   /**
    * @brief Gets the path to the library associated with this factory
+   *
    * @return Library path as a std::string
    */
-  std::string getAssociatedLibraryPath();
+  const std::string & getAssociatedLibraryPath() const;
 
   /**
    * @brief Sets the path to the library associated with this factory
    */
-  void setAssociatedLibraryPath(std::string library_path);
+  void setAssociatedLibraryPath(const std::string & library_path);
 
   /**
    * @brief Associates a ClassLoader owner with this factory,
+   *
    * @param loader Handle to the owning ClassLoader.
    */
   void addOwningClassLoader(ClassLoader * loader);
 
   /**
    * @brief Removes a ClassLoader that is an owner of this factory
+   *
    * @param loader Handle to the owning ClassLoader.
    */
   void removeOwningClassLoader(const ClassLoader * loader);
 
   /**
    * @brief Indicates if the factory is within the usable scope of a ClassLoader
+   *
    * @param loader Handle to the owning ClassLoader.
+   * @return True if the factory is within the usable scope of a ClassLoader, false otherwise
    */
-  bool isOwnedBy(const ClassLoader * loader);
+  bool isOwnedBy(const ClassLoader * loader) const;
 
   /**
    * @brief Indicates if the factory is within the usable scope of any ClassLoader
+   *
+   * @return true if the factory is within the usable scope of any ClassLoader, false otherwise
    */
-  bool isOwnedByAnybody();
+  bool isOwnedByAnybody() const;
 
   /**
-   * A vector of class loaders that own this metaobject
+   * @brief Get the number of associated class Loaders
+   *
+   * @return number of associated class loaders
    */
-  ClassLoaderVector getAssociatedClassLoaders();
+  size_t getAssociatedClassLoadersCount() const;
+
+  /**
+   * @brief Get an associated ClassLoader pointer by index
+   *
+   * @param[in] index The index of the ClassLoader.
+   * @return The ClassLoader pointer or undefined behaviour if the index is out of bounds
+   */
+  ClassLoader * getAssociatedClassLoader(size_t index) const;
 
 protected:
   /**
@@ -130,17 +149,14 @@ protected:
    */
   virtual void dummyMethod() {}
 
-protected:
-  ClassLoaderVector associated_class_loaders_;
-  std::string associated_library_path_;
-  std::string base_class_name_;
-  std::string class_name_;
-  std::string typeid_base_class_name_;
+  AbstractMetaObjectBaseImpl * impl_;
 };
 
 /**
  * @class AbstractMetaObject
- * @brief Abstract base class for factories where polymorphic type variable indicates base class for plugin interface.
+ * @brief Abstract base class for factories where polymorphic type variable indicates base class for
+ *  plugin interface.
+ *
  * @parm B The base class interface for the plugin
  */
 template<class B>
@@ -149,16 +165,17 @@ class AbstractMetaObject : public AbstractMetaObjectBase
 public:
   /**
    * @brief A constructor for this class
+   *
    * @param name The literal name of the class.
    */
   AbstractMetaObject(const std::string & class_name, const std::string & base_class_name)
-  : AbstractMetaObjectBase(class_name, base_class_name)
+  : AbstractMetaObjectBase(class_name, base_class_name, typeid(B).name())
   {
-    AbstractMetaObjectBase::typeid_base_class_name_ = std::string(typeid(B).name());
   }
 
   /**
    * @brief Defines the factory interface that the MetaObject must implement.
+   *
    * @return A pointer of parametric type B to a newly created object.
    */
   virtual B * create() const = 0;
@@ -174,6 +191,7 @@ private:
 /**
  * @class MetaObject
  * @brief The actual factory.
+ *
  * @parm C The derived class (the actual plugin)
  * @parm B The base class interface for the plugin
  */
@@ -190,7 +208,9 @@ public:
   }
 
   /**
-   * @brief The factory interface to generate an object. The object has type C in reality, though a pointer of the base class type is returned.
+   * @brief The factory interface to generate an object. The object has type C in reality, though a
+   * pointer of the base class type is returned.
+   *
    * @return A pointer to a newly created plugin with the base class type (type parameter B)
    */
   B * create() const
