@@ -28,6 +28,7 @@
  */
 
 #include <string>
+#include <vector>
 
 #include "class_loader/meta_object.hpp"
 #include "class_loader/class_loader.hpp"
@@ -37,16 +38,30 @@ namespace class_loader
 namespace impl
 {
 
-AbstractMetaObjectBase::AbstractMetaObjectBase(
-  const std::string & class_name, const std::string & base_class_name)
-: associated_library_path_("Unknown"),
-  base_class_name_(base_class_name),
-  class_name_(class_name),
-  typeid_base_class_name_("UNSET")
+typedef std::vector<class_loader::ClassLoader *> ClassLoaderVector;
+
+class AbstractMetaObjectBaseImpl
 {
+public:
+  ClassLoaderVector associated_class_loaders_;
+  std::string associated_library_path_;
+  std::string base_class_name_;
+  std::string class_name_;
+  std::string typeid_base_class_name_;
+};
+
+AbstractMetaObjectBase::AbstractMetaObjectBase(
+  const std::string & class_name, const std::string & base_class_name,
+  const std::string & typeid_base_class_name)
+: impl_(new AbstractMetaObjectBaseImpl())
+{
+  impl_->associated_library_path_ = "Unknown";
+  impl_->base_class_name_ = base_class_name;
+  impl_->class_name_ = class_name;
+  impl_->typeid_base_class_name_ = typeid_base_class_name;
   CONSOLE_BRIDGE_logDebug(
-    "class_loader.impl.AbstractMetaObjectBase: "
-    "Creating MetaObject %p (base = %s, derived = %s, library path = %s)",
+    "class_loader.impl.AbstractMetaObjectBase: Creating MetaObject %p "
+    "(base = %s, derived = %s, library path = %s)",
     this, baseClassName().c_str(), className().c_str(), getAssociatedLibraryPath().c_str());
 }
 
@@ -56,36 +71,37 @@ AbstractMetaObjectBase::~AbstractMetaObjectBase()
     "class_loader.impl.AbstractMetaObjectBase: "
     "Destroying MetaObject %p (base = %s, derived = %s, library path = %s)",
     this, baseClassName().c_str(), className().c_str(), getAssociatedLibraryPath().c_str());
+  delete impl_;
 }
 
-std::string AbstractMetaObjectBase::className() const
+const std::string & AbstractMetaObjectBase::className() const
 {
-  return class_name_;
+  return impl_->class_name_;
 }
 
-std::string AbstractMetaObjectBase::baseClassName() const
+const std::string & AbstractMetaObjectBase::baseClassName() const
 {
-  return base_class_name_;
+  return impl_->base_class_name_;
 }
 
-std::string AbstractMetaObjectBase::typeidBaseClassName() const
+const std::string & AbstractMetaObjectBase::typeidBaseClassName() const
 {
-  return typeid_base_class_name_;
+  return impl_->typeid_base_class_name_;
 }
 
-std::string AbstractMetaObjectBase::getAssociatedLibraryPath()
+const std::string & AbstractMetaObjectBase::getAssociatedLibraryPath() const
 {
-  return associated_library_path_;
+  return impl_->associated_library_path_;
 }
 
-void AbstractMetaObjectBase::setAssociatedLibraryPath(std::string library_path)
+void AbstractMetaObjectBase::setAssociatedLibraryPath(const std::string & library_path)
 {
-  associated_library_path_ = library_path;
+  impl_->associated_library_path_ = library_path;
 }
 
 void AbstractMetaObjectBase::addOwningClassLoader(ClassLoader * loader)
 {
-  ClassLoaderVector & v = associated_class_loaders_;
+  ClassLoaderVector & v = impl_->associated_class_loaders_;
   if (std::find(v.begin(), v.end(), loader) == v.end()) {
     v.push_back(loader);
   }
@@ -93,29 +109,35 @@ void AbstractMetaObjectBase::addOwningClassLoader(ClassLoader * loader)
 
 void AbstractMetaObjectBase::removeOwningClassLoader(const ClassLoader * loader)
 {
-  ClassLoaderVector & v = associated_class_loaders_;
+  ClassLoaderVector & v = impl_->associated_class_loaders_;
   ClassLoaderVector::iterator itr = std::find(v.begin(), v.end(), loader);
   if (itr != v.end()) {
     v.erase(itr);
   }
 }
 
-bool AbstractMetaObjectBase::isOwnedBy(const ClassLoader * loader)
+bool AbstractMetaObjectBase::isOwnedBy(const ClassLoader * loader) const
 {
-  ClassLoaderVector & v = associated_class_loaders_;
-  ClassLoaderVector::iterator itr = std::find(v.begin(), v.end(), loader);
-  return itr != v.end();
+  const ClassLoaderVector & v = impl_->associated_class_loaders_;
+  auto it = std::find(v.begin(), v.end(), loader);
+  return it != v.end();
 }
 
-bool AbstractMetaObjectBase::isOwnedByAnybody()
+bool AbstractMetaObjectBase::isOwnedByAnybody() const
 {
-  return associated_class_loaders_.size() > 0;
+  return impl_->associated_class_loaders_.size() > 0;
 }
 
-ClassLoaderVector AbstractMetaObjectBase::getAssociatedClassLoaders()
+size_t AbstractMetaObjectBase::getAssociatedClassLoadersCount() const
 {
-  return associated_class_loaders_;
+  return impl_->associated_class_loaders_.size();
+}
+
+ClassLoader * AbstractMetaObjectBase::getAssociatedClassLoader(size_t index) const
+{
+  return impl_->associated_class_loaders_[index];
 }
 
 }  // namespace impl
+
 }  // namespace class_loader
