@@ -36,31 +36,46 @@
 class Dog : public Base
 {
 public:
-  virtual void saySomething() {std::cout << "Bark" << std::endl;}
+  void saySomething()
+  {
+    printf("Bark\n");
+  }
 };
 
 class Cat : public Base
 {
 public:
-  virtual void saySomething() {std::cout << "Meow" << std::endl;}
+  void saySomething()
+  {
+    printf("Meow\n");
+  }
 };
 
 class Duck : public Base
 {
 public:
-  virtual void saySomething() {std::cout << "Quack" << std::endl;}
+  void saySomething()
+  {
+    printf("Quack\n");
+  }
 };
 
 class Cow : public Base
 {
 public:
-  virtual void saySomething() {std::cout << "Moooo" << std::endl;}
+  void saySomething()
+  {
+    printf("Moooo\n");
+  }
 };
 
 class Sheep : public Base
 {
 public:
-  virtual void saySomething() {std::cout << "Baaah" << std::endl;}
+  void saySomething()
+  {
+    printf("Baaah\n");
+  }
 };
 
 CLASS_LOADER_REGISTER_CLASS(Dog, Base)
