@@ -36,25 +36,37 @@
 class Robot : public Base
 {
 public:
-  virtual void saySomething() {std::cout << "Beep boop" << std::endl;}
+  void saySomething()
+  {
+    printf("Beep boop\n");
+  }
 };
 
 class Alien : public Base
 {
 public:
-  virtual void saySomething() {std::cout << "Znornoff!!!" << std::endl;}
+  void saySomething()
+  {
+    printf("Znornoff!!!\n");
+  }
 };
 
 class Monster : public Base
 {
 public:
-  virtual void saySomething() {std::cout << "BEAAAHHHH" << std::endl;}
+  void saySomething()
+  {
+    printf("BEAAAHHHH\n");
+  }
 };
 
 class Zombie : public Base
 {
 public:
-  virtual void saySomething() {std::cout << "Brains!!!" << std::endl;}
+  void saySomething()
+  {
+    printf("Brains!!!\n");
+  }
 };
 
 
